import { EnumeratorLike } from "@reactive-js/enumerable";
import { Option } from "@reactive-js/option";
import { Operator } from "@reactive-js/pipe";
export declare type CharCode = number;
export interface CharStreamLike extends EnumeratorLike<void, CharCode> {
    index: number;
    readonly src: string;
}
export declare type Parser<T> = {
    (input: CharStreamLike): T;
};
export declare const throwParseError: <T>(charStream: CharStreamLike) => T;
export declare const isParseError: (e: unknown) => boolean;
export declare const createCharStream: (input: string) => CharStreamLike;
export declare function concat<TA, TB>(a: Parser<TA>, b: Parser<TB>): Parser<[TA, TB]>;
export declare function concat<TA, TB, TC>(a: Parser<TA>, b: Parser<TB>, c: Parser<TC>): Parser<[TA, TB, TC]>;
export declare function concat<TA, TB, TC, TD>(a: Parser<TA>, b: Parser<TB>, c: Parser<TC>, d: Parser<TD>): Parser<[TA, TB, TC, TD]>;
export declare function concat<TA, TB, TC, TD, TE>(a: Parser<TA>, b: Parser<TB>, c: Parser<TC>, d: Parser<TD>, e: Parser<TE>): Parser<[TA, TB, TC, TD, TE]>;
export declare function concat<TA, TB, TC, TD, TE, TF>(a: Parser<TA>, b: Parser<TB>, c: Parser<TC>, d: Parser<TD>, e: Parser<TE>, f: Parser<TF>): Parser<[TA, TB, TC, TD, TE, TF]>;
export declare function concat<TA, TB, TC, TD, TE, TF, TG>(a: Parser<TA>, b: Parser<TB>, c: Parser<TC>, d: Parser<TD>, e: Parser<TE>, f: Parser<TF>, g: Parser<TG>): Parser<[TA, TB, TC, TD, TE, TF, TG]>;
export declare function concat<TA, TB, TC, TD, TE, TF, TG, TH>(a: Parser<TA>, b: Parser<TB>, c: Parser<TC>, d: Parser<TD>, e: Parser<TE>, f: Parser<TF>, g: Parser<TG>, h: Parser<TH>): Parser<[TA, TB, TC, TD, TE, TF, TG, TH]>;
export declare function concat<TA, TB, TC, TD, TE, TF, TG, TH, TI>(a: Parser<TA>, b: Parser<TB>, c: Parser<TC>, d: Parser<TD>, e: Parser<TE>, f: Parser<TF>, g: Parser<TG>, h: Parser<TH>, i: Parser<TI>): Parser<[TA, TB, TC, TD, TE, TF, TG, TH, TI]>;
export declare const followedBy: <T>(other: Parser<unknown>) => Operator<Parser<T>, Parser<T>>;
export declare const map: <TA, TB>(mapper: (result: TA) => TB) => Operator<Parser<TA>, Parser<TB>>;
export declare const mapTo: <TA, TB>(v: TB) => Operator<Parser<TA>, Parser<TB>>;
export declare const parseWithOrThrow: <T>(parser: Parser<T>) => Operator<string, T>;
export declare const parseWith: <T>(parse: Parser<T>) => Operator<string, Option<T>>;
export declare const or: <TA, TB>(otherParse: Parser<TB>) => Operator<Parser<TA>, Parser<TA | TB>>;
export declare const many: <T>(options?: {
    min?: number | undefined;
    max?: number | undefined;
}) => Operator<Parser<T>, Parser<readonly T[]>>;
export declare const manyIgnore: <T>(options?: {
    min?: number | undefined;
    max?: number | undefined;
}) => Operator<Parser<T>, Parser<void>>;
export declare const optional: <T>(parse: Parser<T>) => Parser<Option<T>>;
export declare const orDefault: <T>(default_: () => T) => Operator<Parser<Option<T>>, Parser<T>>;
export declare const sepBy1: <T>(separator: Parser<unknown>) => Operator<Parser<T>, Parser<readonly T[]>>;
export declare const sepBy: <T>(separator: Parser<unknown>) => Operator<Parser<T>, Parser<readonly T[]>>;
export declare const string: (str: string) => Parser<string>;
export declare const satisfy: (f: (char: number) => boolean) => Parser<number>;
export declare const manySatisfy: (options?: {
    min?: number | undefined;
    max?: number | undefined;
}) => Operator<Parser<number>, Parser<string>>;
export declare const char: (c: string) => Parser<number>;
export declare const pEof: (charStream: CharStreamLike) => void;
export declare const pSemicolon: Parser<number>;
export declare const pComma: Parser<number>;
export declare const pSpace: Parser<number>;
export declare const pColon: Parser<number>;
export declare const pPeriod: Parser<number>;
export declare const pEquals: Parser<number>;
export declare const pForwardSlash: Parser<number>;
export declare const pDash: Parser<number>;
export declare const pOpenParen: Parser<number>;
export declare const pCloseParen: Parser<number>;
export declare const pDquote: Parser<number>;
export declare const pAsterisk: Parser<number>;
//# sourceMappingURL=index.d.ts.map